import { Prompt } from "makitso-prompt";
import { Stores } from "../plugins/stores";
import { Commands, ContextSchema } from "../types";
/**
 * Get an instance of Context
 *
 * @param {Object} arg0 -
 * @param {Object} arg0.commands - Commands definition.
 * @param {Object} arg0.schema - Context schema.
 * @param {Object} arg0.stores - Object of store objects.
 * @returns {Object} An instance of Context.
 */
declare type ContextArgs = {
    commands: Commands;
    prompt?: Prompt;
    schema: ContextSchema;
    stores: Stores;
};
export declare function Context(arg0: ContextArgs): {
    commands: Commands;
    schema: ContextSchema;
    stores: Stores;
    prompt: Prompt | undefined;
    /**
     * Get a context property
     *
     * @param {string} prop - The property to get.
     * @returns {Promise} - The value of the property
     */
    get: (prop: string) => Promise<unknown>;
    /**
     * Set a context property
     *
     * @param {string} prop - The property to set.
     * @param {*} value - The value of the property.
     * @returns {Promise} - The value of the property
     */
    set: (prop: string, value: unknown) => Promise<unknown>;
    /**
     * Delete a property from context
     *
     * @param {string} prop - The property to delete.
     * @param {string} [variant="default"] - The name of the variant of this property type.
     * @returns {Promise} - The value of the property
     */
    delete: (prop: string) => Promise<unknown>;
    /**
     * get a store object
     *
     * @param {String} store - a store identifier
     * @returns {Object} store
     */
    getStore: (store: string) => Promise<import("../plugins/stores").Store>;
    /**
     * list available store identifiers
     *
     * @returns {String[]} store identifiers
     */
    listStores: () => Promise<string[]>;
    /**
     * get the schema
     * @returns {Object} a copy of the schema plain object
     */
    getSchema: () => any;
};
export {};
