"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Context = void 0;
var app_error_1 = __importDefault(require("@lecstor/app-error"));
var get_1 = __importDefault(require("lodash/get"));
var isFunction_1 = __importDefault(require("lodash/isFunction"));
var makitso_prompt_1 = require("makitso-prompt");
var keypress_autocomplete_1 = require("./keypress-autocomplete");
/**
 * @param {String} schemaPath - the dotted path to the property schema
 * @returns {Error} - PropertySchemaNotDefined
 */
function PropNotSetError(schemaPath) {
    return app_error_1.default("PropertySchemaNotDefined", {
        status: 400,
        message: "There is no schema defined for the property \"" + schemaPath + "\"",
        meta: { propSchemaPath: schemaPath },
    });
}
/**
 * @param {String} propRef - the dotted path to the property
 * @returns {Error} - PropBadFormatError
 */
function PropBadFormatError(propRef) {
    return app_error_1.default("PropertyBadlyFormatted", {
        status: 400,
        message: "\"Property " + propRef + "\" is invalid.\n    A property reference is a dotted string with exactly two or three parts.",
        meta: { propRef: propRef },
    });
}
/**
 * @param {String} storeRef - a store idetifier
 * @returns {Error} - StoreNotFoundError
 */
function StoreNotFoundError(storeRef) {
    return app_error_1.default("StoreNotFound", {
        status: 400,
        message: "The requested context store \"" + storeRef + "\" does not exist.",
        meta: { storeRef: storeRef },
    });
}
/**
 * @param {String} propRef - the dotted path to the property
 * @returns {Object} - property meta
 */
function splitProp(propRef) {
    var splitPropRef = propRef.split(".");
    if (splitPropRef.length < 2 || splitPropRef.length > 3) {
        throw PropBadFormatError(propRef);
    }
    var plugin = splitPropRef[0], property = splitPropRef[1], _a = splitPropRef[2], variant = _a === void 0 ? "default" : _a;
    var schemaPath = [plugin, property].join(".");
    var propertyPath = [schemaPath, variant].join(".");
    return { schemaPath: schemaPath, propertyPath: propertyPath, variant: variant };
}
/**
 * @typedef {Object} PropertyMeta
 * @property {String} store - a store identifier
 * @property {Object} prompt - a question definition
 * @property {String} prompt.type - question type
 * @property {String} prompt.name - question name
 * @property {String} prompt.message - the question
 * @property {String} schemaPath - dotted path to the property schema
 * @property {String} propertyPath - dotted path to the property in the store
 * @property {String} variant - the difference between the schemaPath and the propertyPath
 */
/**
 *
 * @param {Object} schema - context schema
 * @param {String} propRef - the dotted path to the property
 * @returns {PropertyMeta} property metadata
 */
function getPropMeta(schema, propRef) {
    var propMeta = splitProp(propRef);
    var propSchema = get_1.default(schema, propMeta.schemaPath);
    if (!propSchema) {
        throw PropNotSetError(propMeta.schemaPath);
    }
    return __assign(__assign({}, propSchema), propMeta);
}
function Context(arg0) {
    var commands = arg0.commands, schema = arg0.schema, stores = arg0.stores, prompt = arg0.prompt;
    return {
        commands: commands,
        schema: schema,
        stores: stores,
        prompt: prompt,
        /**
         * Get a context property
         *
         * @param {string} prop - The property to get.
         * @returns {Promise} - The value of the property
         */
        get: function (prop) {
            return __awaiter(this, void 0, void 0, function () {
                var meta, variant, _a, mDefault, footer, header, _b, maskInput, mPrompt, storedValueIs, suggest, value, prompt, suggestList, _c, complete, thisPrompt, answer;
                return __generator(this, function (_d) {
                    switch (_d.label) {
                        case 0:
                            meta = getPropMeta(this.schema, prop);
                            if (!meta.ask) {
                                return [2 /*return*/, this.stores[meta.store].get(meta)];
                            }
                            variant = (meta.variant !== "default" ? meta.variant : false) || "";
                            _a = meta.ask, mDefault = _a.default, footer = _a.footer, header = _a.header, _b = _a.maskInput, maskInput = _b === void 0 ? false : _b, mPrompt = _a.prompt, storedValueIs = _a.storedValueIs, suggest = _a.suggest;
                            return [4 /*yield*/, this.stores[meta.store].get(meta)];
                        case 1:
                            value = _d.sent();
                            // use stored value without prompting
                            if (value && storedValueIs === "response") {
                                return [2 /*return*/, value];
                            }
                            prompt = this.prompt || new makitso_prompt_1.Prompt();
                            if (!suggest) return [3 /*break*/, 5];
                            if (!isFunction_1.default(suggest)) return [3 /*break*/, 3];
                            return [4 /*yield*/, suggest({ property: prop })];
                        case 2:
                            _c = _d.sent();
                            return [3 /*break*/, 4];
                        case 3:
                            _c = suggest;
                            _d.label = 4;
                        case 4:
                            suggestList = _c;
                            complete = keypress_autocomplete_1.keyPressAutoComplete(suggestList);
                            Object.assign(prompt, {
                                keyPressers: __spreadArrays(prompt.keyPressers, [complete]),
                            });
                            _d.label = 5;
                        case 5:
                            thisPrompt = {
                                header: header ? header.replace("{variant}", variant) : "",
                                prompt: mPrompt ? mPrompt.replace("{variant}", variant) : "",
                                footer: footer ? footer.replace("{variant}", variant) : "",
                                default: value && storedValueIs === "default" ? value : mDefault,
                                maskInput: maskInput,
                            };
                            return [4 /*yield*/, prompt.start(thisPrompt)];
                        case 6:
                            answer = _d.sent();
                            return [4 /*yield*/, this.set(prop, answer)];
                        case 7:
                            _d.sent();
                            return [2 /*return*/, answer];
                    }
                });
            });
        },
        /**
         * Set a context property
         *
         * @param {string} prop - The property to set.
         * @param {*} value - The value of the property.
         * @returns {Promise} - The value of the property
         */
        set: function (prop, value) {
            return __awaiter(this, void 0, void 0, function () {
                var meta;
                return __generator(this, function (_a) {
                    meta = getPropMeta(this.schema, prop);
                    if (!this.stores[meta.store]) {
                        throw StoreNotFoundError(meta.store);
                    }
                    return [2 /*return*/, this.stores[meta.store].set(meta, value)];
                });
            });
        },
        /**
         * Delete a property from context
         *
         * @param {string} prop - The property to delete.
         * @param {string} [variant="default"] - The name of the variant of this property type.
         * @returns {Promise} - The value of the property
         */
        delete: function (prop) {
            return __awaiter(this, void 0, void 0, function () {
                var meta;
                return __generator(this, function (_a) {
                    meta = getPropMeta(this.schema, prop);
                    return [2 /*return*/, this.stores[meta.store].delete(meta)];
                });
            });
        },
        /**
         * get a store object
         *
         * @param {String} store - a store identifier
         * @returns {Object} store
         */
        getStore: function (store) {
            return __awaiter(this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    return [2 /*return*/, this.stores[store]];
                });
            });
        },
        /**
         * list available store identifiers
         *
         * @returns {String[]} store identifiers
         */
        listStores: function () {
            return __awaiter(this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    return [2 /*return*/, Object.keys(this.stores)];
                });
            });
        },
        /**
         * get the schema
         * @returns {Object} a copy of the schema plain object
         */
        getSchema: function () {
            return JSON.parse(JSON.stringify(this.schema));
        },
    };
}
exports.Context = Context;
