import { PropertyMeta } from "./memory-store";
import { Store } from "./types";
export declare type FileStoreArgs = {
    path: string;
    data?: {
        [key: string]: unknown;
    };
};
/**
 * get a FileStore instance
 *
 * @param {Object} args -
 * @param {String} args.file - the path to a JSON file
 * @returns {Object} FileStore
 */
export declare class FileStore implements Store {
    filePath: string;
    initData: {
        [key: string]: unknown;
    };
    constructor(args: FileStoreArgs);
    load(): Promise<this>;
    /**
     * get a property from the store
     *
     * @param {Object} prop - property metadata
     * @param {String} prop.propertyPath - the path to the property
     * @returns {Promise} property value
     */
    get(prop: PropertyMeta): Promise<unknown>;
    /**
     * set a property in the store
     *
     * @param {Object} prop - property metadata
     * @param {String} prop.propertyPath - the path to the property
     * @param {*} value - The value to set on the property.
     * @returns {Promise} property value
     */
    set(prop: PropertyMeta, value: unknown): Promise<unknown>;
    /**
     * delete a property from the store
     *
     * @param {Object} prop - property metadata
     * @param {String} prop.propertyPath - the path to the property
     * @returns {Promise} previous property value
     */
    delete(prop: PropertyMeta): Promise<unknown>;
    /**
     * Read file contents
     *
     * @returns {Promise} the value of the property
     */
    read(): Promise<any>;
}
export declare function plugin(args: FileStoreArgs): Promise<{
    stores: {
        file: FileStore;
    };
}>;
