import { Store } from "./types";
declare type PropertyMeta = {
    schemaPath: string;
    variant?: string;
    storeOptions: {
        service: string;
        account: string;
    };
};
export declare class KeychainStore implements Store {
    /**
     * Get the value of a property from the store.
     *
     * @param {Object} prop - The property schema.
     * @param {String} prop.schemaPath - Dotted path to the property in the schema.
     * @param {String} prop.variant - The name of the variant of the property.
     * @param {Object} prop.storeOptions - Property options for the store.
     * @param {String} prop.storeOptions.service - The service name for the property.
     * @param {String} prop.storeOptions.account - The account name for the property.
     * @returns {Promise} the property value
     */
    get({ schemaPath, variant, storeOptions }: PropertyMeta): Promise<string | null>;
    /**
     * Set the value of a property in the store.
     *
     * @param {Object} prop - The property schema.
     * @param {String} prop.schemaPath - Dotted path to the property in the schema.
     * @param {String} prop.variant - The name of the variant of the property.
     * @param {Object} prop.storeOptions - Property options for the store.
     * @param {String} prop.storeOptions.service - The service name for the property.
     * @param {String} prop.storeOptions.account - The account name for the property.
     * @param {String} value - The value to set on the property.
     * @returns {Promise} the property value
     */
    set(prop: PropertyMeta, value: string): Promise<string>;
    /**
     * Delete a property from the store.
     *
     * @param {Object} prop - The property schema.
     * @param {String} prop.schemaPath - Dotted path to the property in the schema.
     * @param {String} prop.variant - The name of the variant of the property.
     * @param {Object} prop.storeOptions - Property options for the store.
     * @param {String} prop.storeOptions.service - The service name for the property.
     * @param {String} prop.storeOptions.account - The account name for the property.
     * @returns {Promise} the property value
     */
    delete(prop: PropertyMeta): Promise<boolean>;
}
export declare function plugin(): {
    stores: {
        keychain: KeychainStore;
    };
};
export {};
