"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugin = exports.KeychainStore = void 0;
var keytar_1 = __importDefault(require("keytar"));
/**
 * Given a list of strings, replace {variant} with the value of variant
 *
 * @param {String} variant - The name of the variant of the property.
 * @param {...String} values - store option values
 * @returns {Array} The service and account values.
 */
function replaceVariant(variant) {
    var values = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        values[_i - 1] = arguments[_i];
    }
    return values.map(function (value) { return value.replace("{variant}", variant); });
}
/**
 * Extract store options from prop object.
 *
 * If store options are not provided then the prop name and variant are used as
 * the service and account values.
 *
 * @param {Object} prop - The property schema with name.
 * @param {String} prop.schemaPath - Dotted path to the property in the schema.
 * @param {String} prop.variant - The name of the variant of the property.
 * @param {Object} prop.storeOptions - Property options for the store.
 * @param {String} prop.storeOptions.service - The service name for the property.
 * @param {String} prop.storeOptions.account - The account name for the property.
 * @returns {Array} The service and account values.
 */
function extractProp(_a) {
    var schemaPath = _a.schemaPath, _b = _a.variant, variant = _b === void 0 ? "" : _b, storeOptions = _a.storeOptions;
    var _c = storeOptions || {}, _d = _c.service, service = _d === void 0 ? schemaPath : _d, _e = _c.account, account = _e === void 0 ? variant : _e;
    return replaceVariant(variant, service, account);
}
var KeychainStore = /** @class */ (function () {
    function KeychainStore() {
    }
    /**
     * Get the value of a property from the store.
     *
     * @param {Object} prop - The property schema.
     * @param {String} prop.schemaPath - Dotted path to the property in the schema.
     * @param {String} prop.variant - The name of the variant of the property.
     * @param {Object} prop.storeOptions - Property options for the store.
     * @param {String} prop.storeOptions.service - The service name for the property.
     * @param {String} prop.storeOptions.account - The account name for the property.
     * @returns {Promise} the property value
     */
    KeychainStore.prototype.get = function (_a) {
        var schemaPath = _a.schemaPath, variant = _a.variant, storeOptions = _a.storeOptions;
        var _b = extractProp({
            schemaPath: schemaPath,
            variant: variant,
            storeOptions: storeOptions,
        }), service = _b[0], account = _b[1];
        return keytar_1.default.getPassword(service, account);
    };
    /**
     * Set the value of a property in the store.
     *
     * @param {Object} prop - The property schema.
     * @param {String} prop.schemaPath - Dotted path to the property in the schema.
     * @param {String} prop.variant - The name of the variant of the property.
     * @param {Object} prop.storeOptions - Property options for the store.
     * @param {String} prop.storeOptions.service - The service name for the property.
     * @param {String} prop.storeOptions.account - The account name for the property.
     * @param {String} value - The value to set on the property.
     * @returns {Promise} the property value
     */
    KeychainStore.prototype.set = function (prop, value) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, service, account;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = extractProp(prop), service = _a[0], account = _a[1];
                        return [4 /*yield*/, keytar_1.default.setPassword(service, account, value)];
                    case 1:
                        _b.sent();
                        return [2 /*return*/, value];
                }
            });
        });
    };
    /**
     * Delete a property from the store.
     *
     * @param {Object} prop - The property schema.
     * @param {String} prop.schemaPath - Dotted path to the property in the schema.
     * @param {String} prop.variant - The name of the variant of the property.
     * @param {Object} prop.storeOptions - Property options for the store.
     * @param {String} prop.storeOptions.service - The service name for the property.
     * @param {String} prop.storeOptions.account - The account name for the property.
     * @returns {Promise} the property value
     */
    KeychainStore.prototype.delete = function (prop) {
        var _a = extractProp(prop), service = _a[0], account = _a[1];
        return keytar_1.default.deletePassword(service, account);
    };
    return KeychainStore;
}());
exports.KeychainStore = KeychainStore;
function plugin() {
    return {
        stores: {
            keychain: new KeychainStore(),
        },
    };
}
exports.plugin = plugin;
