import { Store } from "./types";
export declare type PropertyMeta = {
    propertyPath: string;
};
export declare type MemoryStoreArgs = {
    data: Data;
};
declare type Data = {
    [key: string]: unknown;
};
export declare class MemoryStore implements Store {
    data: Data;
    constructor(args?: MemoryStoreArgs);
    /**
     * get a property from the store
     *
     * @param {Object} prop - property metadata
     * @param {String} prop.propertyPath - the path to the property
     * @returns {Promise} property value
     */
    get(prop: PropertyMeta): Promise<unknown>;
    /**
     * set a property in the store
     *
     * @param {Object} prop - property metadata
     * @param {String} prop.propertyPath - the path to the property
     * @param {*} value - The value to set on the property.
     * @returns {Promise} property value
     */
    set(prop: PropertyMeta, value: unknown): Promise<unknown>;
    /**
     * delete a property from the store
     *
     * @param {Object} prop - property metadata
     * @param {String} prop.propertyPath - the path to the property
     * @returns {Promise} previous property value
     */
    delete(prop: PropertyMeta): Promise<unknown>;
    /**
     * read the store data
     *
     * @returns {Object} the store data
     */
    read(): any;
}
export declare function plugin(args: MemoryStoreArgs): {
    stores: {
        session: MemoryStore;
    };
};
export {};
